"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class Hook {
    constructor(source, name, callable) {
        this.source = source;
        this.name = name;
        this.callable = callable;
    }
    fire(args) {
        return this.callable(args);
    }
}
exports.Hook = Hook;
class HookEngine {
    constructor() {
        this.hooks = new Map();
    }
    register(source, hook, listener) {
        const h = new Hook(source, hook, listener);
        this.getRegistered(hook).push(h);
    }
    fire(hook, args) {
        return __awaiter(this, void 0, void 0, function* () {
            const registeredHooks = this.hooks.get(hook) || [];
            return Promise.all(registeredHooks.map((h) => h.fire(args)));
        });
    }
    getSources(hook) {
        return [...new Set(this.getRegistered(hook).map(h => h.source))];
    }
    hasSources(hook, sources) {
        return sources.filter(s => this.getSources(hook).includes(s)).length > 0;
    }
    getRegistered(hook) {
        let registeredHooks = this.hooks.get(hook);
        if (!registeredHooks) {
            registeredHooks = [];
            this.hooks.set(hook, registeredHooks);
        }
        return registeredHooks;
    }
}
exports.HookEngine = HookEngine;
