"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const errors_1 = require("./errors");
const shell_1 = require("./utils/shell");
exports.ERROR_SHELL_COMMAND_NOT_FOUND = 'SHELL_COMMAND_NOT_FOUND';
class Shell {
    run(command, args, _a) {
        var { showExecution = true, showError = true, fatalOnNotFound = true, fatalOnError = true, truncateErrorOutput } = _a, crossSpawnOptions = __rest(_a, ["showExecution", "showError", "fatalOnNotFound", "fatalOnError", "truncateErrorOutput"]);
        return __awaiter(this, void 0, void 0, function* () {
            const fullCmd = command + ' ' + (args.length > 0 ? args.join(' ') : '');
            const truncatedCmd = fullCmd.length > 50 ? fullCmd.substring(0, 70) + '...' : fullCmd;
            if (showExecution) {
                console.log(`\n> ${chalk.green(fullCmd)}\n`);
            }
            try {
                const out = yield shell_1.runcmd(command, args, crossSpawnOptions);
                if (crossSpawnOptions.stdio === 'inherit') {
                    process.stdout.write('\n');
                }
                return out;
            }
            catch (e) {
                if (e.code === 'ENOENT') {
                    if (fatalOnNotFound) {
                        throw new errors_1.FatalException(`Command not found: ${chalk.green(command)}`, 127);
                    }
                    else {
                        throw exports.ERROR_SHELL_COMMAND_NOT_FOUND;
                    }
                }
                if (!Array.isArray(e)) {
                    throw e;
                }
                let [code, err] = e;
                if (truncateErrorOutput && err && err.length > truncateErrorOutput) {
                    err = `${chalk.bold('(truncated)')} ... ` + err.substring(err.length - truncateErrorOutput);
                }
                if (fatalOnError) {
                    if (showError) {
                        const helpLine = crossSpawnOptions.stdio === 'inherit' ? '.\n' : (err ? `:\n\n${err}` : ' with no output.\n');
                        throw new errors_1.FatalException(`An error occurred while running ${chalk.green(truncatedCmd)} (exit code ${code})` + helpLine, code);
                    }
                    else {
                        throw new errors_1.FatalException(`Subprocess (${chalk.green(command)}) encountered an error (exit code ${code}).`, code);
                    }
                }
                throw e;
            }
        });
    }
}
exports.Shell = Shell;
