"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const string_1 = require("./utils/string");
exports.validators = {
    required(input, key) {
        if (!input) {
            if (key) {
                return `${chalk.bold(key)} must not be empty.`;
            }
            else {
                return 'Must not be empty.';
            }
        }
        return true;
    },
    email(input, key) {
        if (!string_1.isValidEmail(input)) {
            if (key) {
                return `${chalk.bold(key)} is an invalid email address.`;
            }
            else {
                return 'Invalid email address.';
            }
        }
        return true;
    }
};
function combine(...validators) {
    return function (input) {
        for (let v of validators) {
            let o = v(input);
            if (o !== true) {
                return o;
            }
        }
        return true;
    };
}
exports.combine = combine;
function contains(...values) {
    return function (input, key) {
        if (values.indexOf(input) === -1) {
            if (key) {
                return `${chalk.bold(key)} must be one of: (${values.map(v => chalk.bold(v)).join(', ')})`;
            }
            else {
                return `Must be one of: (${values.map(v => chalk.bold(v)).join(', ')})`;
            }
        }
        return true;
    };
}
exports.contains = contains;
